Ray3

Struct Ray3 

#[repr(C)]
pub struct Ray3<T> { pub origin: Point3<T>, pub direction: Vec3<T>, pub distance: Interval<T>, }
Ray3 structure.

Rays are typically used to trace shapes for intersection tests. See Ray3::trace for more information.

Fields

origin: Point3<T>

The origin point where the ray starts.

direction: Vec3<T>

The direction in which the ray extends from its origin.

This vector should be normalized and non-zero; otherwise, results may be incorrect.

distance: Interval<T>

Distance limit.

Implementations

Source

impl<T> Ray3<T>

pub fn new( origin: Point3<T>, direction: Vec3<T>, distance: Interval<T>, ) -> Ray3<T>
where T: Float,

Constructs a new ray with normalized direction.

The direction is normalized. Zero directions may result in unexpected behavior.

Source

impl<T: Float> Ray3<T>

pub fn at(&self, distance: T) -> Point3<T>

Returns the point at a given distance along the ray’s direction.

pub fn reflect(&self, hit: &Hit3<T>) -> Ray3<T>

Reflects the ray at the given hit point.

Source

impl<T: Float> Ray3<T>

pub fn inside<U: Trace3<T> + ?Sized>(&self, shape: &U) -> bool

Returns if the ray starts inside the shape.

pub fn inside_collection<Shape: Trace3<T>, I: IntoIterator<Item = Shape>>( &self, shapes: I, ) -> bool

Returns if the ray starts inside any shape in the collection.

pub fn trace<U: Trace3<T> + ?Sized>(&self, shape: &U) -> Option<Hit3<T>>

Trace the ray against a shape.

pub fn trace_collection<Shape: Trace3<T>, I: IntoIterator<Item = Shape>>( &self, shapes: I, ) -> Option<Hit3<T>>

Trace the ray against a collection of shapes.